package prestamorecursos.mundo;

import java.util.Date;

import listas.modelo.*;
import listas.modelo.PosicionIlegalException;

public class Universidad {
	private Lista<Estudiante> estudiantes;
	private Lista<Recurso> recursos;
	private Lista<Prestamo> prestamos;

	public Universidad() {
		estudiantes = new Lista<Estudiante>();
		recursos = new Lista<Recurso>();
		prestamos = new Lista<Prestamo>();

	}

	public boolean agregarEstudiante(int codigo, String nombre, String email,
			Date fechaNac, String sexo, String programa)
			throws PosicionIlegalException {
		Estudiante est = buscarEstudiante(codigo);
		if (est == null) {
			estudiantes.agregar(new Estudiante(codigo, nombre, email, fechaNac,
					sexo, programa));
			return true;
		}
		return false;

	}

	public Estudiante buscarEstudiante(int codigo)
			throws PosicionIlegalException {
		for (int i = 0; i < estudiantes.getTamanio(); i++) {
			Estudiante est = estudiantes.getValor(i);
			if (est.getCodigo() == codigo) {
				return est;
			}
		}
		return null;
	}

	public boolean agregarRecurso(int id, String nombre)
			throws PosicionIlegalException {
		Recurso rec = buscarRecurso(id);
		if (rec == null) {
			recursos.agregar(new Recurso(id, nombre, true));
			return true;
		}
		return false;

	}

	public Recurso buscarRecurso(int id) throws PosicionIlegalException {
		for (int i = 0; i < recursos.getTamanio(); i++) {
			Recurso rec = recursos.getValor(i);
			if (rec.getId() == id) {
				return rec;
			}
		}
		return null;
	}

	public boolean prestarRecurso(int codigo, int id)
			throws PosicionIlegalException {
		Estudiante est = buscarEstudiante(codigo);
		Recurso rec = buscarRecurso(id);
		if (est == null || rec == null) {
			System.out.println("Estudiante o recurso no existen");
			return false;
		}

		if (!rec.isDisponible()) {
			System.out.println("El recurso " + rec
					+ " ya esta prestado a otro estudiante");
			return false;
		}

		Prestamo pres = new Prestamo(rec, est);
		pres.setFechaPrestamo(new Date());
		rec.setDisponible(false);
		prestamos.agregar(pres);
		System.out.println("El recurso " + rec + " fue prestamo al estudiante "
				+ est);
		return true;
	}

	public Estudiante consultarEstudianteTieneRecurso(int id)
			throws PosicionIlegalException {
		Recurso rec = buscarRecurso(id);
		if (rec == null) {
			System.out.println("Recurso no existen");
			return null;
		}

		if (rec.isDisponible()) {
			System.out.println("El recurso " + rec + " esta disponible");
			return null;
		}

		for (int i = 0; i < prestamos.getTamanio(); i++) {
			Prestamo pres = prestamos.getValor(i);

			if (rec.getId() == id && !rec.isDisponible()) {
				System.out.println("El recurso " + rec
						+ " esta prestado al estudiante "
						+ pres.getEstudiante());
				return pres.getEstudiante();
			}

		}
		return null;
	}

	public Lista<Recurso> consultarRecursosDeUnEstudiante(int codigo)
			throws PosicionIlegalException {
		Estudiante est = buscarEstudiante(codigo);
		if (est == null) {
			System.out.println("Estudiante no existen");
			return null;
		}

		Lista<Recurso> recursosprestados = new Lista<Recurso>();

		for (int i = 0; i < prestamos.getTamanio(); i++) {
			Prestamo pres = prestamos.getValor(i);

			if (pres.getEstudiante().getCodigo() == codigo
					&& !pres.getRecurso().isDisponible()) {
				recursosprestados.agregar(pres.getRecurso());
			}

		}
		return recursosprestados;
	}

	public boolean devolverRecurso(int id) throws PosicionIlegalException {
		Recurso rec = buscarRecurso(id);
		if (rec == null) {
			System.out.println("Recurso no existen");
			return false;
		}

		if (rec.isDisponible()) {
			System.out.println("El recurso " + rec
					+ " no se puede devolver porque esta disponible");
			return false;
		}
		for (int i = 0; i < prestamos.getTamanio(); i++) {
			Prestamo pres = prestamos.getValor(i);

			if (pres.getRecurso().getId() == id
					&& pres.getFechaDevolucion() == null) {
				pres.setFechaDevolucion(new Date());
				rec.setDisponible(true);
				System.out.println("Se ha devuelvo el recurso " + rec
						+ " por parte del estudiante " + pres.getEstudiante()
						+ " satisfactoriamente!");
				return true;
			}

		}

		return false;

	}

	public boolean eliminarRecurso(int id) throws PosicionIlegalException {
		Recurso rec = buscarRecurso(id);
		if (rec == null) {
			System.out.println("Recurso no existen");
			return false;
		}
		// Borrar el recurso de los prestamos
		for (int i = 0; i < prestamos.getTamanio(); i++) {
			Prestamo pres = prestamos.getValor(i);

			if (pres.getRecurso().getId() == id) {
				prestamos.remover(i);
			}

		}
		
		//Borrar el recurso de recursos
		for (int i = 0; i < recursos.getTamanio(); i++) {

			if (recursos.getValor(i).getId() == id) {
				recursos.remover(i);
			}

		}		
		System.out.println("El recurso " + rec + "fue eliminado satisfactoriamente");
		return true;
	}

	public boolean eliminarEstudiante(int codigo) throws PosicionIlegalException {
		Estudiante est = buscarEstudiante(codigo);
		if (est == null) {
			System.out.println("Estudiante no existe");
			return false;
		}
		// Borrar el estudiante de los prestamos
		for (int i = 0; i < prestamos.getTamanio(); i++) {
			Prestamo pres = prestamos.getValor(i);

			if (pres.getEstudiante().getCodigo() == codigo) {
				prestamos.remover(i);
			}

		}
		
		//Borrar el estudiante de estudiantes
		for (int i = 0; i < estudiantes.getTamanio(); i++) {

			if (estudiantes.getValor(i).getCodigo() == codigo) {
				estudiantes.remover(i);
			}

		}		
		System.out.println("El estudiante " + est + " fue eliminado satisfactoriamente");
		return true;
	}

}
